@@ -1,7 +1,8 @@
-/* $Id: parser_misc.c 412 2007-02-27 02:29:16Z jrjohansen $ */
+/* $Id: parser_misc.c 1042 2007-11-29 18:06:53Z jrjohansen $ */
 
 /*
- *   Copyright (c) 1999, 2000, 2002, 2003, 2004, 2005 NOVELL (All rights reserved)
+ *   Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
+ *   NOVELL (All rights reserved)
  *
  *   This program is free software; you can redistribute it and/or
  *   modify it under the terms of version 2 of the GNU General Public
@@ -28,6 +29,7 @@
 #include <netinet/in.h>
 #include <sys/socket.h>
 #include <arpa/inet.h>
+#include <linux/capability.h>
 
 #include "parser.h"
 #include "parser_yacc.h"
@@ -46,59 +48,45 @@ struct keyword_table {
 };
 
 static struct keyword_table keyword_table[] = {
-	/* capabilities */
-	{"capability",		TOK_CAPABILITY},
-	{"chown",		TOK_CAP_CHOWN},
-	{"dac_override",	TOK_CAP_DAC_OVERRIDE},
-	{"dac_read_search",	TOK_CAP_DAC_READ_SEARCH},
-	{"fowner",		TOK_CAP_FOWNER},
-	{"fsetid",		TOK_CAP_FSETID},
-	{"kill",		TOK_CAP_KILL},
-	{"setgid",		TOK_CAP_SETGID},
-	{"setuid",		TOK_CAP_SETUID},
-	{"setpcap",		TOK_CAP_SETPCAP},
-	{"linux_immutable",	TOK_CAP_LINUX_IMMUTABLE},
-	{"net_bind_service",	TOK_CAP_NET_BIND_SERVICE},
-	{"net_broadcast",	TOK_CAP_NET_BROADCAST},
-	{"net_admin",		TOK_CAP_NET_ADMIN},
-	{"net_raw",		TOK_CAP_NET_RAW},
-	{"ipc_lock",		TOK_CAP_IPC_LOCK},
-	{"ipc_owner",		TOK_CAP_IPC_OWNER},
-	{"sys_module",		TOK_CAP_SYS_MODULE},
-	{"sys_rawio",		TOK_CAP_SYS_RAWIO},
-	{"sys_chroot",		TOK_CAP_SYS_CHROOT},
-	{"sys_ptrace",		TOK_CAP_SYS_PTRACE},
-	{"sys_pacct",		TOK_CAP_SYS_PACCT},
-	{"sys_admin",		TOK_CAP_SYS_ADMIN},
-	{"sys_boot",		TOK_CAP_SYS_BOOT},
-	{"sys_nice",		TOK_CAP_SYS_NICE},
-	{"sys_resource",	TOK_CAP_SYS_RESOURCE},
-	{"sys_time",		TOK_CAP_SYS_TIME},
-	{"sys_tty_config",	TOK_CAP_SYS_TTY_CONFIG},
-	{"mknod",		TOK_CAP_MKNOD},
-	{"lease",		TOK_CAP_LEASE},
-	{"audit_write",		TOK_CAP_AUDIT_WRITE},
-	{"audit_control",	TOK_CAP_AUDIT_CONTROL},
 	/* flags */
 	{"flags",		TOK_FLAGS},
-	{"debug",		TOK_FLAG_DEBUG},
-	{"complain",		TOK_FLAG_COMPLAIN},
-	{"audit",		TOK_FLAG_AUDIT},
 	/* network */
-	{"via",			TOK_VIA},
-	{"tcp_connect",		TOK_TCP_CONN},
-	{"tcp_accept",		TOK_TCP_ACPT},
-	{"tcp_connected",	TOK_TCP_CONN_ESTB},
-	{"tcp_accepted",	TOK_TCP_ACPT_ESTB},
-	{"udp_send",		TOK_UDP_SEND},
-	{"udp_receive",		TOK_UDP_RECV},
-	{"to",			TOK_TO},
-	{"from",		TOK_FROM},
+	{"network",		TOK_NETWORK},
 	/* misc keywords */
+	{"capability",		TOK_CAPABILITY},
 	{"if",			TOK_IF},
 	{"else",		TOK_ELSE},
 	{"not",			TOK_NOT},
 	{"defined",		TOK_DEFINED},
+	{"change_profile",	TOK_CHANGE_PROFILE},
+	{"unsafe",		TOK_UNSAFE},
+	{"link",		TOK_LINK},
+	{"owner",		TOK_OWNER},
+	/* terminate */
+	{NULL, 0}
+};
+
+/* for alpha matches, check for keywords */
+static int get_table_token(const char *name, struct keyword_table *table,
+			   const char *keyword)
+{
+	int i;
+
+	for (i = 0; table[i].keyword; i++) {
+		PDEBUG("Checking %s %s\n", name, table[i].keyword);
+		if (strcmp(keyword, table[i].keyword) == 0) {
+			PDEBUG("Found %s %s\n", name, table[i].keyword);
+			return table[i].token;
+		}
+	}
+
+	PDEBUG("Unable to find %s %s\n", name, keyword);
+	return -1;
+}
+
+static struct keyword_table capability_table[] = {
+	/* capabilities */
+	#include "cap_names.h"
 	/* terminate */
 	{NULL, 0}
 };
@@ -106,19 +94,161 @@ static struct keyword_table keyword_table[] = {
 /* for alpha matches, check for keywords */
 int get_keyword_token(const char *keyword)
 {
-	int i;
+	return get_table_token("keyword", keyword_table, keyword);
+}
 
-	for (i = 0; keyword_table[i].keyword; i++) {
-		PDEBUG("Checking keyword %s\n", keyword_table[i].keyword);
-		if (strcmp(keyword, keyword_table[i].keyword) == 0) {
-			PDEBUG("Found keyword %s\n", keyword_table[i].keyword);
-			return keyword_table[i].token;
+int name_to_capability(const char *keyword)
+{
+	return get_table_token("capability", capability_table, keyword);
+}
+
+static struct keyword_table address_family[] = {
+/*	{"unix",	AF_UNIX},
+	{"local",	AF_LOCAL},	*/
+	{"inet",	AF_INET},
+/*	{"ax25",	AF_AX25},
+	{"ipx",		AF_IPX},
+	{"appletalk",	AF_APPLETALK},
+	{"netrom",	AF_NETROM},
+	{"bridge",	AF_BRIDGE},
+	{"atmpvc",	AF_ATMPVC},
+	{"x25",		AF_X25}, */
+	{"inet6",	AF_INET6},
+/*	{"rose",	AF_ROSE},
+	{"decnet",	AF_DECnet},
+	{"netbeui",	AF_NETBEUI},
+	{"security",	AF_SECURITY},
+	{"key",		AF_KEY},
+	{"netlink",	AF_NETLINK},
+	{"route",	AF_ROUTE},
+	{"packet",	AF_PACKET},
+	{"ash",		AF_ASH},
+	{"econet",	AF_ECONET},
+	{"atmsvc",	AF_ATMSVC},
+	{"sna",		AF_SNA},
+	{"irda",	AF_IRDA},
+	{"pppox",	AF_PPPOX},
+	{"wanpipe",	AF_WANPIPE},
+	{"llc",		AF_LLC},
+	{"tipc",	AF_TIPC},
+	{"bluetooth",	AF_BLUETOOTH},
+	{"iucv",	AF_IUCV},
+	{"rxrpc",	AF_RXRPC}, */
+	/* terminate */
+	{NULL, 0}
+};
+
+struct network_tuple {
+	char *family_name;
+	unsigned int family;
+	char *type_name;
+	unsigned int type;
+	char *protocol_name;
+	unsigned int protocol;
+};
+
+/* FIXME: currently just treating as a bit mask this will have to change
+ * set up a table of mappings, there can be several mappings for a
+ * given match.
+ * currently the mapping does not set the protocol for stream/dgram to
+ * anything other than 0.
+ *   network inet tcp -> network inet stream 0 instead of
+ *   network inet raw tcp.
+ * some entries are just provided for completeness at this time
+ */
+/* values stolen from /etc/protocols - needs to change */
+#define RAW_TCP 6
+#define RAW_UDP 17
+#define RAW_ICMP 1
+#define RAW_ICMPv6 58
+
+/* used by af_name.h to auto generate table entries for "name", AF_NAME
+ * pair */
+#define AA_GEN_NET_ENT(name, AF) {name, AF, "stream", SOCK_STREAM, "", 0xffffff}, {name, AF, "dgram", SOCK_DGRAM, "", 0xffffff}, {name, AF, "seqpacket", SOCK_SEQPACKET, "", 0xffffff}, {name, AF, "rdm", SOCK_RDM, "", 0xffffff}, {name, AF, "raw", SOCK_RAW, "", 0xffffff}, {name, AF, "packet", SOCK_PACKET, "", 0xffffff},
+/*FIXME: missing {name, AF, "dccp", SOCK_DCCP, "", 0xfffffff}, */
+
+static struct network_tuple network_mappings[] = {
+	/* basic types */
+	#include "af_names.h"
+/* FIXME: af_names.h is missing AF_LLC, AF_TIPC */
+	/* mapped types */
+	{"inet",	AF_INET,	"raw",		SOCK_RAW,
+	 "tcp",	        1 << RAW_TCP},
+	{"inet",	AF_INET,	"raw",		SOCK_RAW,
+	 "udp",		1 << RAW_UDP},
+	{"inet",	AF_INET,	"raw",		SOCK_RAW,
+	 "icmp",	1 << RAW_ICMP},
+	{"inet",	AF_INET,	"tcp",		SOCK_STREAM,
+	 "",		0xffffffff},	/* should we give raw tcp too? */
+	{"inet",	AF_INET,	"udp",		SOCK_DGRAM,
+	 "",		0xffffffff},	/* should these be open masks? */
+	{"inet",	AF_INET,	"icmp",		SOCK_RAW,
+	 "",		1 << RAW_ICMP},
+	{"inet6",	AF_INET6,	"tcp",		SOCK_STREAM,
+	 "",		0xffffffff},
+	{"inet6",	AF_INET6,	"udp",		SOCK_DGRAM,
+	 "",		0xffffffff},
+/* what do we do with icmp on inet6?
+	{"inet6",	AF_INET,	"icmp",		SOCK_RAW,	0},
+	{"inet6",	AF_INET,	"icmpv6",	SOCK_RAW,	0},
+*/
+	/* terminate */
+	{NULL, 0, NULL, 0, NULL, 0}
+};
+
+struct aa_network_entry *new_network_ent(unsigned int family,
+					 unsigned int type,
+					 unsigned int protocol)
+{
+	struct aa_network_entry *new_entry;
+	new_entry = calloc(1, sizeof(struct aa_network_entry));
+	if (new_entry) {
+		new_entry->family = family;
+		new_entry->type = type;
+		new_entry->protocol = protocol;
+		new_entry->next = NULL;
+	}
+	return new_entry;
+}
+
+struct aa_network_entry *network_entry(const char *family, const char *type,
+				       const char *protocol)
+{
+	int i;
+	struct aa_network_entry *new_entry, *entry = NULL;
+
+	for (i = 0; network_mappings[i].family_name; i++) {
+		if (family) {
+			PDEBUG("Checking family %s\n", network_mappings[i].family_name);
+			if (strcmp(family, network_mappings[i].family_name) != 0)
+				continue;
+			PDEBUG("Found family %s\n", family);
 		}
+		if (type) {
+			PDEBUG("Checking type %s\n", network_mappings[i].type_name);
+			if (strcmp(type, network_mappings[i].type_name) != 0)
+				continue;
+			PDEBUG("Found type %s\n", type);
+		}
+		if (protocol) {
+			PDEBUG("Checking protocol type %s\n", network_mappings[i].protocol_name);
+			if (strcmp(type, network_mappings[i].protocol_name) != 0)
+				continue;
+			/* fixme should we allow specifying protocol by #
+			 * without needing the protocol mapping? */
+		}
+		/* if here we have a match */
+		new_entry = new_network_ent(network_mappings[i].family,
+					    network_mappings[i].type,
+					    network_mappings[i].protocol);
+		if (!new_entry)
+			yyerror(_("Memory allocation error."));
+		new_entry->next = entry;
+		entry = new_entry;
 	}
 
-	PDEBUG("Unable to find keyword %s\n", keyword);
-	return -1;
-}
+	return entry;
+};
 
 char *processunquoted(char *string, int len)
 {
@@ -273,108 +403,123 @@ static void warn_uppercase(void)
 		warned_uppercase = 1;
 	}
 }
-int parse_mode(const char *str_mode)
-{
-	/* The 'check' int is a bit of a kludge, but we need some context
-	   when we're doing permission checking */
 
-#define IS_DIFF_QUAL(q) (qual && qual != (q) ? TRUE : (qual = (q), FALSE))
+static int parse_sub_mode(const char *str_mode, const char *mode_desc)
+{
+
+#define IS_DIFF_QUAL(mode, q) (((mode) & AA_MAY_EXEC) && (((mode) & (AA_EXEC_MODIFIERS | AA_EXEC_UNSAFE)) != (q)))
 
 	int mode = 0;
 	const char *p;
-	char qual = 0;
 
 	PDEBUG("Parsing mode: %s\n", str_mode);
 
+	if (!str_mode)
+		return 0;
+
 	p = str_mode;
 	while (*p) {
 		char this = *p;
 		char next = *(p + 1);
 		char lower;
+		int tmode = 0;
 
 reeval:
 		switch (this) {
 		case COD_READ_CHAR:
-			PDEBUG("Parsing mode: found READ\n");
+			PDEBUG("Parsing mode: found %s READ\n", mode_desc);
 			mode |= AA_MAY_READ;
 			break;
 
 		case COD_WRITE_CHAR:
-			PDEBUG("Parsing mode: found WRITE\n");
-			mode |= AA_MAY_WRITE;
+			PDEBUG("Parsing mode: found %s WRITE\n", mode_desc);
+			if ((mode & AA_MAY_APPEND) && !(mode & AA_MAY_WRITE))
+				yyerror(_("Conflict 'a' and 'w' perms are mutually exclusive."));
+			mode |= AA_MAY_WRITE | AA_MAY_APPEND;
+			break;
+
+		case COD_APPEND_CHAR:
+			PDEBUG("Parsing mode: found %s APPEND\n", mode_desc);
+			if (mode & AA_MAY_WRITE)
+				yyerror(_("Conflict 'a' and 'w' perms are mutually exclusive."));
+			mode |= AA_MAY_APPEND;
 			break;
 
 		case COD_LINK_CHAR:
-			PDEBUG("Parsing mode: found LINK\n");
+			PDEBUG("Parsing mode: found %s LINK\n", mode_desc);
 			mode |= AA_MAY_LINK;
 			break;
 
+		case COD_LOCK_CHAR:
+			PDEBUG("Parsing mode: found %s LOCK\n", mode_desc);
+			mode |= AA_MAY_LOCK;
+			break;
+
 		case COD_INHERIT_CHAR:
 			PDEBUG("Parsing mode: found INHERIT\n");
-			if (next != COD_EXEC_CHAR && tolower(next) != COD_EXEC_CHAR) {
-				yyerror(_("Exec qualifier 'i' must be followed by 'x'"));
-			} else if (IS_DIFF_QUAL(this)) {
+			if (IS_DIFF_QUAL(mode, AA_EXEC_INHERIT)) {
 				yyerror(_("Exec qualifier 'i' invalid, conflicting qualifier already specified"));
 			} else {
 				if (next != tolower(next))
 					warn_uppercase();
-				mode |=
-				    (AA_EXEC_INHERIT | AA_MAY_EXEC);
+				mode |= (AA_EXEC_INHERIT | AA_MAY_EXEC);
 				p++;	/* skip 'x' */
 			}
 			break;
 
-		case COD_UNSAFE_UNCONSTRAINED_CHAR:
-			mode |= AA_EXEC_UNSAFE;
-			pwarn(_("Unconstrained exec qualifier (%c%c) allows some dangerous environment variables "
+		case COD_UNSAFE_UNCONFINED_CHAR:
+			tmode = AA_EXEC_UNSAFE;
+			pwarn(_("Unconfined exec qualifier (%c%c) allows some dangerous environment variables "
 				"to be passed to the unconfined process; 'man 5 apparmor.d' for details.\n"),
-			      COD_UNSAFE_UNCONSTRAINED_CHAR, COD_EXEC_CHAR);
+			      COD_UNSAFE_UNCONFINED_CHAR, COD_EXEC_CHAR);
 			/* fall through */
-		case COD_UNCONSTRAINED_CHAR:
-			PDEBUG("Parsing mode: found UNCONSTRAINED\n");
-			if (next != COD_EXEC_CHAR && tolower(next) != COD_EXEC_CHAR) {
-				yyerror(_("Exec qualifier '%c' must be followed by 'x'"),
-					this);
-			} else if (IS_DIFF_QUAL(this)) {
+		case COD_UNCONFINED_CHAR:
+			PDEBUG("Parsing mode: found UNCONFINED\n");
+			if (IS_DIFF_QUAL(mode, tmode | AA_EXEC_UNCONFINED)) {
 				yyerror(_("Exec qualifier '%c' invalid, conflicting qualifier already specified"),
 					this);
 			} else {
 				if (next != tolower(next))
 					warn_uppercase();
-				mode |=
-				    (AA_EXEC_UNCONSTRAINED |
-				     AA_MAY_EXEC);
+				mode |= tmode | AA_EXEC_UNCONFINED |
+				    AA_MAY_EXEC;
 				p++;	/* skip 'x' */
 			}
+			tmode = 0;
 			break;
 
 		case COD_UNSAFE_PROFILE_CHAR:
-			mode |= AA_EXEC_UNSAFE;
+			tmode = AA_EXEC_UNSAFE;
 			/* fall through */
 		case COD_PROFILE_CHAR:
 			PDEBUG("Parsing mode: found PROFILE\n");
-			if (next != COD_EXEC_CHAR && tolower(next) != COD_EXEC_CHAR) {
-				yyerror(_("Exec qualifier '%c' must be followed by 'x'"),
-					this);
-			} else if (IS_DIFF_QUAL(this)) {
+			if (tolower(next) == COD_INHERIT_CHAR) {
+				if (IS_DIFF_QUAL(mode, tmode | AA_EXEC_PROFILE_OR_INHERIT)) {
+					yyerror(_("Exec qualifier '%c%c' invalid, conflicting qualifier already specified"), this, next);
+				} else {
+					mode |= tmode | AA_MAY_EXEC |
+					    AA_EXEC_PROFILE_OR_INHERIT;
+					p += 2;		/* skip x */
+				}
+			} else if (IS_DIFF_QUAL(mode, tmode | AA_EXEC_PROFILE)) {
 				yyerror(_("Exec qualifier '%c' invalid, conflicting qualifier already specified"),
 					this);
 			} else {
 				if (next != tolower(next))
 					warn_uppercase();
-				mode |=
-				    (AA_EXEC_PROFILE | AA_MAY_EXEC);
+				mode |= tmode | AA_EXEC_PROFILE | AA_MAY_EXEC;
 				p++;	/* skip 'x' */
 			}
+			tmode = 0;
 			break;
 
 		case COD_MMAP_CHAR:
-			PDEBUG("Parsing mode: found MMAP\n");
+			PDEBUG("Parsing mode: found %s MMAP\n", mode_desc);
 			mode |= AA_EXEC_MMAP;
 			break;
 
 		case COD_EXEC_CHAR:
-			PDEBUG("Parsing mode: found EXEC\n");
+			PDEBUG("Parsing mode: found %s EXEC\n", mode_desc);
 			yyerror(_("Invalid mode, 'x' must be preceded by exec qualifier 'i', 'p', or 'u'"));
 			break;
 
@@ -385,6 +530,7 @@ reeval:
 			switch (lower) {
 			case COD_READ_CHAR:
 			case COD_WRITE_CHAR:
+			case COD_APPEND_CHAR:
 			case COD_LINK_CHAR:
 			case COD_INHERIT_CHAR:
 			case COD_MMAP_CHAR:
@@ -410,59 +556,18 @@ reeval:
 	return mode;
 }
 
-struct cod_net_entry *new_network_entry(int action,
-					struct ipv4_endpoints *addrs,
-					char *interface)
+int parse_mode(const char *str_mode)
 {
-	struct cod_net_entry *entry = NULL;
-
-	entry = (struct cod_net_entry *)
-	    			malloc(sizeof(struct cod_net_entry));
-	entry->saddr = (struct in_addr *)malloc(sizeof(struct in_addr));
-	entry->smask = (struct in_addr *)malloc(sizeof(struct in_addr));
-	entry->daddr = (struct in_addr *)malloc(sizeof(struct in_addr));
-	entry->dmask = (struct in_addr *)malloc(sizeof(struct in_addr));
-
-	if (!addrs || !entry || !entry->saddr || !entry->smask ||
-	    !entry->daddr || !entry->dmask) {
-		yyerror(_("Memory allocation error."));
-		return NULL;
-	}
-
-	entry->next = NULL;
-	entry->mode = action;
-	entry->iface = interface ? interface : NULL;
-
-	if (addrs->src) {
-		PDEBUG("Assigning source\n");
-		entry->saddr->s_addr = addrs->src->addr.s_addr & addrs->src->mask;
-		entry->smask->s_addr = addrs->src->mask;
-		entry->src_port[0] = addrs->src->port[0];
-		entry->src_port[1] = addrs->src->port[1];
-	} else {
-		entry->saddr->s_addr = 0;
-		entry->smask->s_addr = 0;
-		entry->src_port[0] = MIN_PORT;
-		entry->src_port[1] = MAX_PORT;
-	}
-
-	if (addrs->dest) {
-		PDEBUG("Assigning source\n");
-		entry->daddr->s_addr = addrs->dest->addr.s_addr & addrs->dest->mask;
-		entry->dmask->s_addr = addrs->dest->mask;
-		entry->dst_port[0] = addrs->dest->port[0];
-		entry->dst_port[1] = addrs->dest->port[1];
-	} else {
-		entry->daddr->s_addr = 0;
-		entry->dmask->s_addr = 0;
-		entry->dst_port[0] = MIN_PORT;
-		entry->dst_port[1] = MAX_PORT;
-	}
-
-	return entry;
+	int tmp, mode = 0;
+	tmp = parse_sub_mode(str_mode, "");
+	mode = SHIFT_MODE(tmp, AA_USER_SHIFT);
+	mode |= SHIFT_MODE(tmp, AA_OTHER_SHIFT);
+	if (mode & ~AA_VALID_PERMS)
+		yyerror(_("Internal error generated invalid perm 0x%llx\n"), mode);
+	return mode;
 }
 
-struct cod_entry *new_entry(char *id, char *mode)
+struct cod_entry *new_entry(char *namespace, char *id, int mode, char *link_id)
 {
 	struct cod_entry *entry = NULL;
 
@@ -470,8 +575,10 @@ struct cod_entry *new_entry(char *id, char *mode)
 	if (!entry)
 		return NULL;
 
-	entry->name = id ? id : NULL;
-	entry->mode = mode ? parse_mode(mode) : 0;
+	entry->namespace = namespace;
+	entry->name = id;
+	entry->link_name = link_id;
+	entry->mode = mode;
 	entry->deny = FALSE;
 
 	entry->pattern_type = ePatternInvalid;
@@ -492,7 +599,9 @@ struct cod_entry *copy_cod_entry(struct cod_entry *orig)
 	if (!entry)
 		return NULL;
 
+	entry->namespace = orig->namespace ? strdup(orig->namespace) : NULL;
 	entry->name = strdup(orig->name);
+	entry->link_name = orig->link_name ? strdup(orig->link_name) : NULL;
 	entry->mode = orig->mode;
 	entry->deny = orig->deny;
 
@@ -506,25 +615,18 @@ struct cod_entry *copy_cod_entry(struct cod_entry *orig)
 	return entry;
 }
 
-void free_ipv4_endpoints(struct ipv4_endpoints *addrs)
-{
-	if (!addrs)
-		return;
-	if (addrs->src)
-		free(addrs->src);
-	if (addrs->dest)
-		free(addrs->dest);
-	free(addrs);
-}
-
 void free_cod_entries(struct cod_entry *list)
 {
 	if (!list)
 		return;
 	if (list->next)
 		free_cod_entries(list->next);
+	if (list->namespace)
+		free(list->namespace);
 	if (list->name)
 		free(list->name);
+	if (list->link_name)
+		free(list->link_name);
 	if (list->pat.regex)
 		free(list->pat.regex);
 	if (list->pat.compiled)
@@ -532,23 +634,36 @@ void free_cod_entries(struct cod_entry *list)
 	free(list);
 }
 
-void free_net_entries(struct cod_net_entry *list)
+static void debug_base_perm_mask(int mask)
 {
-	if (!list)
-		return;
-	if (list->next)
-		free_net_entries(list->next);
-	if (list->saddr)
-		free(list->saddr);
-	if (list->smask)
-		free(list->smask);
-	if (list->daddr)
-		free(list->daddr);
-	if (list->dmask)
-		free(list->dmask);
-	if (list->iface)
-		free(list->iface);
-	free(list);
+	if (HAS_MAY_READ(mask))
+		printf("%c", COD_READ_CHAR);
+	if (HAS_MAY_WRITE(mask))
+		printf("%c", COD_WRITE_CHAR);
+	if (HAS_MAY_APPEND(mask))
+		printf("%c", COD_APPEND_CHAR);
+	if (HAS_MAY_LINK(mask))
+		printf("%c", COD_LINK_CHAR);
+	if (HAS_MAY_LOCK(mask))
+		printf("%c", COD_LOCK_CHAR);
+	if (HAS_EXEC_INHERIT(mask))
+		printf("%c", COD_INHERIT_CHAR);
+	if (HAS_EXEC_UNCONFINED(mask)) {
+		if (HAS_EXEC_UNSAFE(mask))
+			printf("%c", COD_UNSAFE_UNCONFINED_CHAR);
+		else
+			printf("%c", COD_UNCONFINED_CHAR);
+	}
+	if (HAS_EXEC_PROFILE(mask)) {
+		if (HAS_EXEC_UNSAFE(mask))
+			printf("%c", COD_UNSAFE_PROFILE_CHAR);
+		else
+			printf("%c", COD_PROFILE_CHAR);
+	}
+	if (HAS_EXEC_MMAP(mask))
+		printf("%c", COD_MMAP_CHAR);
+	if (HAS_MAY_EXEC(mask))
+		printf("%c", COD_EXEC_CHAR);
 }
 
 void debug_cod_entries(struct cod_entry *list)
@@ -562,83 +677,25 @@ void debug_cod_entries(struct cod_entry *list)
 			printf("Item is NULL!\n");
 
 		printf("Mode:\t");
-		if (HAS_MAY_READ(item->mode))
-			printf("%c", COD_READ_CHAR);
-		if (HAS_MAY_WRITE(item->mode))
-			printf("%c", COD_WRITE_CHAR);
-		if (HAS_MAY_LINK(item->mode))
-			printf("%c", COD_LINK_CHAR);
-		if (HAS_EXEC_INHERIT(item->mode))
-			printf("%c", COD_INHERIT_CHAR);
-		if (HAS_EXEC_UNCONSTRAINED(item->mode)) {
-			if (HAS_EXEC_UNSAFE(item->mode))
-				printf("%c", COD_UNSAFE_UNCONSTRAINED_CHAR);
-			else
-				printf("%c", COD_UNCONSTRAINED_CHAR);
-		}
-		if (HAS_EXEC_PROFILE(item->mode)) {
-			if (HAS_EXEC_UNSAFE(item->mode))
-				printf("%c", COD_UNSAFE_PROFILE_CHAR);
-			else
-				printf("%c", COD_PROFILE_CHAR);
-		}
-		if (HAS_EXEC_MMAP(item->mode))
-			printf("%c", COD_MMAP_CHAR);
-		if (HAS_MAY_EXEC(item->mode))
-			printf("%c", COD_EXEC_CHAR);
-
+		if (HAS_CHANGE_PROFILE(item->mode))
+			printf(" change_profile");
+		if (HAS_EXEC_UNSAFE(item->mode))
+			printf(" unsafe");
+		debug_base_perm_mask(SHIFT_TO_BASE(item->mode, AA_USER_SHIFT));
+		printf(":");
+		debug_base_perm_mask(SHIFT_TO_BASE(item->mode, AA_OTHER_SHIFT));
 		if (item->name)
 			printf("\tName:\t(%s)\n", item->name);
+
 		else
 			printf("\tName:\tNULL\n");
-	}
-}
 
-void debug_cod_net_entries(struct cod_net_entry *list)
-{
-	struct cod_net_entry *item = NULL;
-	struct in_addr src_addr, dst_addr;
-	unsigned long smask;
-	unsigned long dmask;
+		if (item->namespace)
+			printf("\tNamespace:\t(%s)\n", item->namespace);
 
-	printf("--- NetwerkEntries --- \n");
+		if (AA_LINK_BITS & item->mode)
+			printf("\tlink:\t(%s)\n", item->link_name ? item->link_name : "/**");
 
-	list_for_each(list, item) {
-		if (!item)
-			printf("Item is NULL");
-
-		src_addr.s_addr = item->saddr->s_addr;
-		dst_addr.s_addr = item->daddr->s_addr;
-		smask = ntohl(item->smask->s_addr);
-		dmask = ntohl(item->dmask->s_addr);
-
-		printf("Source IP: %s\n", inet_ntoa(src_addr));
-		printf("Source Port: (%hu) - (%hu)\n", item->src_port[0],
-		       item->src_port[1]);
-		printf("Source netmask: %lx\n", smask);
-		fflush(stdout);
-		printf("Destination IP: %s\n", inet_ntoa(dst_addr));
-		printf("Destination Port: %hu - %hu\n", item->dst_port[0],
-		       item->dst_port[1]);
-		printf("Destination netmask: %lx\n", dmask);
-		fflush(stdout);
-		printf("Mode:\t");
-		if (item->mode & AA_TCP_ACCEPT)
-			printf("TA");
-		if (item->mode & AA_TCP_CONNECT)
-			printf("TC");
-		if (item->mode & AA_TCP_ACCEPTED)
-			printf("Ta");
-		if (item->mode & AA_TCP_CONNECTED)
-			printf("Tc");
-		if (item->mode & AA_UDP_SEND)
-			printf("US");
-		if (item->mode & AA_UDP_RECEIVE)
-			printf("UR");
-		if (item->iface != NULL)
-			printf("\nInterface: %s\n", item->iface);
-
-		printf("\n");
 	}
 }
 
@@ -689,6 +746,9 @@ const char *capability_to_name(unsigned int cap)
 void debug_cod_list(struct codomain *cod)
 {
 	unsigned int i;
+	if (cod->namespace)
+		printf("Namespcae:\t\t%s\n", cod->namespace);
+
 	if (cod->name)
 		printf("Name:\t\t%s\n", cod->name);
 	else
@@ -715,9 +775,6 @@ void debug_cod_list(struct codomain *cod)
 	if (cod->entries)
 		debug_cod_entries(cod->entries);
 
-	if (cod->net_entries)
-		debug_cod_net_entries(cod->net_entries);
-
 	printf("\n");
 	dump_policy_hats(cod);
 }
